﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Media;

namespace WPF.Kapitel12.RoundedRectangle
{

    public class RoundedRectangle : FrameworkElement
    {
        public static readonly DependencyProperty FillProperty =
            DependencyProperty.Register("Fill", typeof(Brush), typeof(RoundedRectangle), new FrameworkPropertyMetadata(null, FrameworkPropertyMetadataOptions.AffectsRender));

        public Brush Fill
        {
            get { return (Brush)GetValue(FillProperty); }
            set { SetValue(FillProperty, value); }
        }

        public static readonly DependencyProperty PenProperty =
            DependencyProperty.Register("Pen", typeof(Pen), typeof(RoundedRectangle), new FrameworkPropertyMetadata(null, FrameworkPropertyMetadataOptions.AffectsRender | FrameworkPropertyMetadataOptions.AffectsMeasure));

        public Pen Pen
        {
            get { return (Pen)GetValue(PenProperty); }
            set { SetValue(PenProperty, value); }
        }

        protected override void OnRender(DrawingContext drawingContext)
        {
            Rect rect = new Rect(new Point(0, 0), RenderSize);
            if (Pen != null)
            {
                rect.Width = Math.Max(0, rect.Width - Pen.Thickness);
                rect.Height = Math.Max(0, rect.Height - Pen.Thickness);
                rect.X = Pen.Thickness / 2;
                rect.Y = Pen.Thickness / 2;
            }
            drawingContext.DrawRoundedRectangle(Fill, Pen, rect, 10, 10);
        }

        protected override Size MeasureOverride(Size availableSize)
        {
            Size desiredSize = new Size(0, 0);
            if (Pen != null)
            {
                desiredSize.Width = Pen.Thickness * 2;
                desiredSize.Height = Pen.Thickness * 2;
            }
            return desiredSize;
        }
    }

}
