﻿using System;
using System.Threading;
using System.ComponentModel;

namespace AsyncDataApplication
{
    
    public class Customer : INotifyPropertyChanged
    {
        private string slowName;
        private string fastName;

        public event PropertyChangedEventHandler PropertyChanged;

        public string SlowName
        {
            get
            {
                Thread.Sleep(2500);
                return slowName;
            }
            set
            {
                if (slowName != value)
                {
                    slowName = value;
                    OnPropertyChanged(new PropertyChangedEventArgs("SlowName"));
                }
            }
        }

        public string FastName
        {
            get
            {
                return fastName;
            }
            set
            {
                if (fastName != value)
                {
                    fastName = value;
                    OnPropertyChanged(new PropertyChangedEventArgs("FastName"));
                }
            }
        }

        public static Customer GetCustomerForSlowDataProvider()
        {
            Customer customer = new Customer();
            customer.SlowName = "Dieser Text erscheint spät";
            return customer;
        }

        public static Customer GetCustomerForSlowProperty()
        {
            Customer customer = new Customer();
            customer.FastName = "Dieser Text erscheint sofort";
            customer.SlowName = "Dieser Text erscheint spät";
            return customer;
        }

        public static Customer GetCustomerForPriorityBinding()
        {
            Customer customer = new Customer();
            customer.FastName = "ca. 30 Datensätze (geschätzt)";
            customer.SlowName = "31 Datensätze (exakt)";
            return customer;
        }

        protected void OnPropertyChanged(PropertyChangedEventArgs e)
        {
            if (PropertyChanged != null)
                PropertyChanged(this, e);
        }
    }

}
